package limax.codec.asn1;

import java.io.ByteArrayOutputStream;
import java.util.Collection;

import limax.codec.Codec;
import limax.codec.CodecException;

public class ASN1BitString extends ASN1Object implements PrimitiveObject, ConstructedObject {
	public static final ASN1ObjectIdentifier oid = new ASN1ObjectIdentifier("2.1.0.0");
	public static final ASN1Tag tag = new ASN1Tag(TagClass.Universal, 3);

	private final ASN1PrimitiveObject obj;

	public ASN1BitString() {
		obj = new ASN1PrimitiveObject(tag);
	}

	public ASN1BitString(ASN1Tag tag) {
		obj = new ASN1PrimitiveObject(tag);
	}

	public ASN1BitString(byte[] value) {
		obj = new ASN1PrimitiveObject(tag, value);
	}

	public ASN1BitString(ASN1Tag tag, byte[] value) {
		obj = new ASN1PrimitiveObject(tag, value);
	}

	@Override
	public void setData(byte[] data) {
		obj.setData(data);
	}

	@Override
	public byte[] getData() {
		return obj.getData();
	}

	@Override
	public ASN1ObjectIdentifier getOID() {
		return oid;
	}

	@Override
	public ASN1Tag getTag() {
		return obj.getTag();
	}

	@Override
	public boolean isEmpty() {
		return obj.isEmpty();
	}

	@Override
	public long renderLength() {
		return obj.renderLength();
	}

	@Override
	public void render(Codec c) throws CodecException {
		obj.render(c);
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder("ASN1BitString [");
		for (byte b : getData())
			sb.append(String.format("%02x", b));
		return sb.append("]").toString();
	}

	@Override
	public void addChildren(Collection<ASN1Object> objects) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		byte lastInitial = 0, data[];
		out.write(lastInitial);
		for (ASN1Object obj : objects) {
			ASN1BitString o = (ASN1BitString) obj;
			data = o.getData();
			lastInitial = data[0];
			out.write(data, 1, data.length - 1);
		}
		data = out.toByteArray();
		data[0] = lastInitial;
		setData(data);
	}
}
